/*  ASIOWmpPlg Windows Media Player Plugin
    Copyright (C) Lev Minkovsky
    
    This file is part of ASIOWmpPlg.

    ASIOWmpPlg is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    ASIOWmpPlg is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with ASIOWmpPlg; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "stdafx.h"
#include "WmpStateMonitorPlugin.h"

/////////////////////////////////////////////////////////////////////////////
// CWmpStateMonitorPlugin::CWmpStateMonitorPlugin
// Constructor

CWmpStateMonitorPlugin::CWmpStateMonitorPlugin()
{
    m_dwAdviseCookie = 0;
}

/////////////////////////////////////////////////////////////////////////////
// CWmpStateMonitorPlugin::~CWmpStateMonitorPlugin
// Destructor

CWmpStateMonitorPlugin::~CWmpStateMonitorPlugin()
{
}

/////////////////////////////////////////////////////////////////////////////
// CWmpStateMonitorPlugin:::FinalConstruct
// Called when an plugin is first loaded. Use this function to do one-time
// intializations that could fail instead of doing this in the constructor,
// which cannot return an error.

HRESULT CWmpStateMonitorPlugin::FinalConstruct()
{
    //If Windows Media Player is not properly shut down (e.g. in case of a system crash),
    //it disables all "optional" plugins. Overwrite this behavior by removing
    //registry keys under the "Health" key 
    CRegKey key;
    const TCHAR wmpHealthRegKey[] = _T("Software\\Microsoft\\MediaPlayer");
    key.Open(HKEY_CURRENT_USER, wmpHealthRegKey);
    key.RecurseDeleteKey("Health");
    return S_OK;
}

/////////////////////////////////////////////////////////////////////////////
// CWmpStateMonitorPlugin:::FinalRelease
// Called when a plugin is unloaded. Use this function to free any
// resources allocated in FinalConstruct.

void CWmpStateMonitorPlugin::FinalRelease()
{
    ReleaseCore();
}

/////////////////////////////////////////////////////////////////////////////
// CWmpStateMonitorPlugin::SetCore
// Set WMP core interface

HRESULT CWmpStateMonitorPlugin::SetCore(IWMPCore *pCore)
{
    HRESULT hr = S_OK;

    // release any existing WMP core interfaces
    ReleaseCore();

    // If we get passed a NULL core, this  means
    // that the plugin is being shutdown.

    if (pCore == NULL)
    {
        return S_OK;
    }

    m_spCore = pCore;

    // connect up the event interface
    CComPtr<IConnectionPointContainer>  spConnectionContainer;

    hr = m_spCore->QueryInterface( &spConnectionContainer );

    if (SUCCEEDED(hr))
    {
        hr = spConnectionContainer->FindConnectionPoint( __uuidof(IWMPEvents), &m_spConnectionPoint );
    }

    if (SUCCEEDED(hr))
    {
        hr = m_spConnectionPoint->Advise( GetUnknown(), &m_dwAdviseCookie );

        if ((FAILED(hr)) || (0 == m_dwAdviseCookie))
        {
            m_spConnectionPoint = NULL;
        }
    }

    return hr;
}

/////////////////////////////////////////////////////////////////////////////
// CWmpStateMonitorPlugin::ReleaseCore
// Release WMP core interfaces

void CWmpStateMonitorPlugin::ReleaseCore()
{
    if (m_spConnectionPoint)
    {
        if (0 != m_dwAdviseCookie)
        {
            m_spConnectionPoint->Unadvise(m_dwAdviseCookie);
            m_dwAdviseCookie = 0;
        }
        m_spConnectionPoint = NULL;
    }

    if (m_spCore)
    {
        m_spCore = NULL;
    }
}

/////////////////////////////////////////////////////////////////////////////
// CWmpStateMonitorPlugin::GetProperty
// Get plugin property

HRESULT CWmpStateMonitorPlugin::GetProperty(const WCHAR *pwszName, VARIANT *pvarProperty)
{
    if (NULL == pvarProperty)
    {
        return E_POINTER;
    }

    return E_NOTIMPL;
}

/////////////////////////////////////////////////////////////////////////////
// CWmpStateMonitorPlugin::SetProperty
// Set plugin property

HRESULT CWmpStateMonitorPlugin::SetProperty(const WCHAR *pwszName, const VARIANT *pvarProperty)
{
    return E_NOTIMPL;
}
